const Koa = require('koa')
const path = require('path')
const serve = require('koa-static')

const argv = require('./src/utils/argv')

const CFonts = require('cfonts')

const chalk = require('chalk')

const version = require('./package.json').version


class Framework{

    /**
     * 初始化项目框架
     * $$: 记录env/config文件夹下的配置信息
     * LIFT_TIME: 用于设置
     */
    constructor(){
        let app =  new Koa()
        app.$$ = {}
        app.LIFT_TIME = Date.now()
        this.ROOT_DIR = process.cwd()

        this.staticPath = []

        global.kk = app
        this.app = app

        this.argv = argv(process.argv.slice(2))
    }

    /**
     * 添加静态资源文件夹
     * @param {String} p 需要映射的文件夹路径
     * @param {serve.Options} option 针对静态资源的配置选项
     * @returns {Framework}
     */
    addStatic(p,option={maxage:1000 * 60 * 60 * 24}){
        if(process.env.NODE_ENV === 'development'){
            option.maxage = 0
        }
        this.staticPath.push({
            path:p,
            option
        })
        return this
    }

    /**
     * 设置视图目录,默认为程序运行目录下的views目录
     * @param {String} p 
     * @returns {Framework}
     */
    setViewPath(p){
        this.app.$$.VIEW_DIR = p
        return this
    }

    /**
     * 设置源代码目录,默认为程序运行目录下的src目录
     * @param {String} p 
     * @returns {Framework}
     */
    setSourcePath(p){
        this.app.$$.SOURCE_DIR = p
        return this
    }

    /**
     * 开启webpack watch
     * @param {webpack.Configuration} cfg 
     * @param {any} webpack 传入webpack
     * @param {any} webpackDevMiddleware 传入webpack-dev-middleware
     * @returns {Framework}
     */
    enableWebpack(cfg,webpack,webpackDevMiddleware=null){
        this.webpackCfg = cfg
        this.webpack = webpack
        this.webpackDevMiddleware = webpackDevMiddleware
        return this
    }

    /**
     * 添加middleware
     * @param {compose.Middleware<Context>} middleware 
     * @returns {Framework}
     */
    use(middleware){
        this.app.use(middleware)
        return this
    }

    /**
     * 配置应用程序
     * @param {*} callback 
     * @returns {Framework}
     */
    setup(callback=null){
        let app = this.app
        app.$$.ROOT_DIR = this.ROOT_DIR

        app.$$.CONFIG_DIR = path.join(this.ROOT_DIR, 'config')
    
        app.$$.SOURCE_DIR = app.$$.SOURCE_DIR || path.join(this.ROOT_DIR, 'src')
    
        app.$$.VIEW_DIR = app.$$.VIEW_DIR || path.join(this.ROOT_DIR, 'views')
    
        if(this.staticPath.length == 0){
            let p = path.join(this.ROOT_DIR,'.tmp/public')
            app.use(serve(p))
        }else{
            for(let p of this.staticPath){
                app.use(serve(p.path,p.option))
            }
        }
        require('./src/setup')(app,callback)

        return this
    }

    get $$(){
        return this.app.$$
    }

    get version(){
        return 201707251428
    }


    /**
     * @private
     */
    compileWebpack(){
        if(this.webpackCfg){
            require('./src/utils/webpack')(this.app,this.webpackCfg,this.webpack,this.webpackDevMiddleware)
        }
    }

    /**
     * 启动监听
     * @param {*} cb 
     */
    startup(cb){
        this.compileWebpack()

        let args = [this.argv.port || this.app.$$.config.port || 6880]
        let hostname = this.argv.hostname || this.app.$$.config.hostname
        if(hostname){
            args.push(hostname)
        }

        var listener = this.app.listen(...args, () => {
            const pretyFont = CFonts.render('KK-KOA-FRAMEWORK',{
                font: 'chrome',
                background: 'transparent',
                lineHeight: 1, 
                space: true,   
            })
            console.log(chalk.green('info:'),pretyFont.string)
            console.log(chalk.green('info:'),chalk.gray(new Date().toLocaleString()))
            console.log(chalk.green('info:'),'Version:',version)
            console.log(chalk.green('info:'),'Listening on port ' + listener.address().port)
            console.log(chalk.green('info:'),'To test your website, visit http://localhost:' + listener.address().port)

            cb && cb(listener)
        })
    }

}

module.exports = Framework

module.exports.passport = require('./src/passport')

let Wrapper = require('./src/utils/wrapper')

module.exports.createMethod = Wrapper.createMethod
module.exports.createMeta = Wrapper.createMeta
